/*:
@target MV MZ
@plugindesc スキルツリーコンフィグ v1.0.1
@author うなぎおおとろ
@url https://raw.githubusercontent.com/unagiootoro/RPGMZ/master/SkillTreeConfig.js

@help
[概要]
スキルツリーを導入するプラグインです。
SPを使ってスキルを習得するスキルツリーを作成することができます。

[使用方法]
■ スキルツリーの設定
スキルツリーの設定は、「SkillTreeConfig.js」ファイルを編集することで行います。
基本的な設定としては、アクターごとにスキルツリーのタイプ(剣スキルや魔法スキルなど)を設定し、
そしてタイプごとにスキルツリーを構築します。
スキルツリーの構築は、スキルの派生設定(ファイアⅠを取得したらファイアⅡが取得可能になるなど)によって行います。

■ SPの入手設定
スキルの習得には、SPが必要となります。
SPの入手方法としては
・戦闘終了による獲得
・レベルアップによるSP獲得
の二通りの設定を行うことができます。

・戦闘終了時に得られるSPの設定方法
敵キャラのメモ欄に
<battleEndGainSp: SP>
の形式で記載します。

・レベルアップによるSP獲得方法の設定
コンフィグの「levelUpGainSp」によって設定を行います。

■ イベントでSPを獲得する方法
スクリプトで
skt_gainSp(アクターID, 獲得するSP値)
と記載することで、該当のアクターが指定したSPを獲得することができます。
例えば、アクターIDが1のアクターが5SPを獲得する場合、
skt_gainSp(1, 5);
と記載します。

■ 割り振った累計SPの取得
skt_totalSp(アクターID, 累計SP格納先変数ID)
と記載することで、該当のアクターが今までに割り振ったSPを指定した変数に代入することができます。
例えば、アクターIDが1のアクターの累計SPをID2の変数に代入する場合、
skt_totalSp(1, 2);
と記載します。

■ スキルリセット
スクリプトで
skt_skillReset(アクターID);
と記載することで、一度習得したスキルをリセットすることができます。
例えば、アクターIDが1のアクターのスキルリセットを行う場合、
skt_skillReset(1);
と記載します。

■ スキルツリータイプの有効/無効
スクリプトで
skt_enableType(アクターID, "タイプ名");
と記載することで、タイプを有効にします。

無効にする場合は、
skt_disableType(アクターID, "タイプ名")
と記載します。

無効にしたタイプは、スキルツリーのタイプ一覧には表示されません。

■ タイプの引継ぎ
特定の条件を満たすとスキルツリーに新たなスキルが追加されるようにしたい場合、「タイプの引継ぎ」を使用します。
例えば、タイプ「下位魔法」を「上位魔法」に変更したい場合、あらかじめ両方のタイプをコンフィグに登録した上で、
「上位魔法」は無効化しておきます。そして、タイプの引継ぎ機能を用いて、「下位魔法」を「上位魔法」に引き継がせます。

タイプの引継ぎを行う場合、スクリプトで
skt_migrationType(アクターID, "引継ぎ元タイプ名", "引継ぎ先タイプ名", リセット有無);
と記載します。リセット有無については、引継ぎ後、引継ぎ元のタイプのスキルツリーをリセットする場合、trueを、
リセットしない場合、falseを指定します。
例えば、アクターIDが1のアクターが、タイプ「下位魔法」を「上位魔法」に引き継がせ、さらにスキルリセットを行う場合、
skt_migrationType(アクターID, "下位魔法", "上位魔法", true);
と記載します。

■ マップからスキルツリーを読み込む
マップからスキルツリーの各スキルの配置座標を読み込むことで、ある程度自由なレイアウトのスキルツリーを
作成することができます。この機能によって設定可能なのはスキルの座標のみであり、スキル間の線はプラグイン側で描画します。

・スキル座標の設定
マップ上のイベントにて、設定を行います。
例えば、"ファイア"というスキルがある場合、スキルを配置したい座標に空のイベントを作成し、
イベントのメモ欄に
ファイア
と記載します。すると、"ファイア"とメモ欄に記載したイベントのXY座標がスキルのXY座標として使用されます。

■ スクリプトからスキルツリーを起動
スクリプトで
skt_open(アクターID);
と記載することで、指定したアクターのスキルツリーを起動することができます。

■ スクリプトからスキルツリーのスキルを取得する
スクリプトで
skt_learn(アクターID, "タイプ名", スキル名, 強制取得有無(省略可));
と記載することで、指定したスキルを取得することができます。
スキル名は「skillTreeInfo」で指定したものを使用してください。
強制取得有無にtrueを設定した場合、習得可能か否かの判定を無視して強制的にスキルを取得します。
また、この場合SPは消費されません。
強制取得有無にfalseを設定した場合は通常通りの方法でスキルを取得します。この場合はSPも消費されます。
なおこの項目については省略可能です。省略した場合はfalseが適用されます。
例: アクターID1のキャラクターで、タイプ「剣技」のスキルツリーの「強撃」を取得する場合
skt_learn(1, "剣技", "強撃", false);
*/

const loadSkillTreeConfig = () => {
return {
// =============================================================
// ●ここからは設定項目です。
// =============================================================

// スキルツリーのタイプの設定を行います。
// skillTreeTypes: [ ～ ]の中にアクターの数だけタイプ設定を追加します。

// タイプ設定は、次の形式で設定します。
// { actorId: アクターのID, types: [タイプ情報1, タイプ情報2, ...] }

// タイプ情報は、次の形式で設定します。
// [タイプ種別, タイプ名, タイプの説明, タイプ有効/無効]
// タイプ種別...スキルの派生設定でタイプを識別するためのユニークな識別子を設定します。
// タイプ名...タイプ一覧のウィンドウに表示するタイプ名を設定します。
// タイプの説明...タイプ一覧のウィンドウに表示するタイプの説明を設定します。
// タイプ有効/無効...タイプを有効にする場合は、trueを、無効にする場合は、falseを指定します。
//                  この項目については、省略可能です。省略した場合、trueが指定されます。
skillTreeTypes: [
    {
        actorId: 1,
        types: [
            ["アンのスキル", "スキル", "スキルを取得します\n←Qキー・Wキー→ でページ送り", true, 76],
            
        ]
    },

    {
        actorId: 2,
        types: [
            ["ラィモネのスキル", "スキル", "スキルを取得します\n←Qキー・Wキー→ でページ送り", true, 76],
          
        ]
    },

    {
        actorId: 3,
        types: [
            ["ミスティのスキル", "スキル", "スキルを取得します\n←Qキー・Wキー→ でページ送り", true, 76],
           
        ]
    },

],

// スキルツリーのマップ読み込み設定を行います。
// 読み込むマップは以下の形式で指定します。
// skillTreeMapId: { skillTreeName1: mapID1, skillTreeName2: mapID2, ... }
// skillTreeName...スキルツリーのタイプ名を指定します。
// mapID...読み込むマップIDを指定します。0の場合は読み込みを行いません。
skillTreeMapId: {
    "下位攻撃魔法": 2,
    "上位攻撃魔法": 2,
    "回復魔法": 3,
    "剣技": 4,
    "格闘技": 5,
},

// 各スキルの情報を登録します。
// skillTreeInfo: [ ～ ]の中に登録するスキル数分のスキル情報の登録を行います。

// スキル情報の登録は次の形式で行います。
// [スキル名, スキルID, 必要SP, アイコン情報]
// スキル名...スキルツリーの派生設定でスキルを一意に識別するための識別子
//            識別子なので、実際のスキル名と一致していなくても問題はありません。
// スキルID...データベース上で該当するスキルのID
// 必要SP...スキルの習得に必要なSP
// アイコン情報については、アイコンを使用するか、任意の画像を使用するかに応じて次の形式で登録します。
//   アイコンを使用する場合 ["icon", iconIndex]
//   iconIndex...使用するアイコンのインデックス
//               iconIndexは省略可能です。省略した場合、スキルに設定されているアイコンが使用されます。
//   画像を使用する場合 ["img", fileName]
//   fileName...画像のファイル名。画像は、「img/pictures」フォルダにインポートしてください。
// なお、アイコン情報については省略可能です。省略した場合、["icon"]が適用されます。
skillTreeInfo: [
    // アンのスキル
    ["強斬り", 12, 1, ],
    ["短剣術Ⅰ", 36, 2, ],
    ["真空刃", 13, 4, ],
    ["短剣術Ⅱ", 37, 3, ],
    ["穿ち断ち", 14, 4,],
    ["短剣術Ⅲ", 38, 5,],
    ["情動", 39, 8,],
    ["目覚め", 40, 2, ],
    ["血刃", 15, 4, ],
    ["ダークⅠ", 20, 3, ],
    ["血の魔法", 41, 3, ],
    ["捨て身の構え", 30, 5, ],
    ["血の慟哭", 16, 6, ],
    ["闇のマナⅠ", 42, 6, ],
    ["ダークⅡ", 21, 5, ],
    ["闇のマナⅡ", 43, 3, ],
    ["ダークオール", 24, 6, ],
    ["ダークⅢ", 22, 7, ],
    ["シェルブランド", 32, 3, ],
    ["ウィークウインド", 31, 3, ],
    ["ハンドオブグレゴリア", 33, 5, ],
    ["パワーシール", 28, 2, ],
    ["スピードシール", 29, 3, ],
    ["斥候術Ⅰ", 44, 3, ],
    ["斥候術Ⅱ", 45, 4, ],
    ["魔力再生", 46, 3, ],
    ["効率化", 47, 3, ],



  // ラィモネのスキル
    ["ファイアⅠ", 82, 1, ],
    ["ファイアⅡ", 83, 4, ],
    ["ファイアⅢ", 84, 5, ],
    ["サンダーⅠ", 86, 2, ],
    ["サンダーⅡ", 87, 4,],
    ["サンダーⅢ", 88, 6,],
    ["アイスブランド", 90, 5,],
    ["プリズムレイ", 92, 5, ],
    ["ヴァーミリオンフレア", 94, 7, ],
    ["チャージ", 96, 2, ],
    ["ヒールⅠ", 97, 3, ],
    ["ヒールⅡ", 98, 5, ],
    ["リヴァイブ", 99, 6, ],
    ["ライフシャワー", 100, 4, ],
    ["ブレイブフォース", 102, 3, ],
    ["アクアベール", 103, 3, ],
    ["火のマナⅠ", 106, 2, ],
    ["火のマナⅡ", 107, 3, ],
    ["火のマナⅢ", 108, 3, ],
    ["魔術の心得Ⅰ", 109, 3, ],
    ["魔術の心得Ⅱ", 110, 4, ],
    ["雷のマナⅠ", 111, 2, ],
    ["雷のマナⅡ", 112, 3, ],
    ["雷のマナⅢ", 113, 3, ],
    ["魔術の心得Ⅲ", 114, 4, ],
    ["魔術の心得Ⅳ", 115, 4, ],
    ["光のマナⅠ", 116, 2, ],
    ["光のマナⅡ", 117, 3, ],
    ["光のマナⅢ", 118, 4, ],
    ["水のマナⅠ", 119, 3, ],
    ["水のマナⅡ", 120, 4, ],

 // ミスティのスキル
    ["ドラゴンクロー", 122, 1, ],
    ["超再生", 123, 2, ],
    ["ドラゴンバッシュ", 126, 4, ],
    ["ドラゴンラッシュ", 127, 6, ],
    ["アイスブレス", 128, 4,],
    ["ミストブレイク", 129, 5,],
    ["激龍化", 132, 8,],
    ["かばう", 137, 4, ],
    ["龍の守護", 138, 3, ],
    ["ミストヒール", 139, 3, ],
    ["マナキュアー", 140, 5, ],
    ["スタンテイル", 146, 3, ],
    ["ミストファング", 147, 6, ],
    ["霧の領域", 149, 6, ],
    ["臨戦態勢Ⅰ", 161, 2, ],
    ["臨戦態勢Ⅱ", 162, 3, ],
    ["龍の力", 163, 4, ],
    ["魔力増加", 164, 3, ],
    ["マナの権化", 165, 4, ],
    ["活性Ⅰ", 166, 3, ],
    ["活性Ⅱ", 167, 3, ],
    ["龍の堅鱗", 168, 4, ],
    ["防御姿勢Ⅰ", 169, 3, ],
    ["防御姿勢Ⅱ", 170, 3, ],
    ["霧の衣", 171, 4, ],
    ["回避性能Ⅰ", 172, 2, ],
     ["先手のコツⅠ", 173, 3, ],
    ["再生力", 174, 4, ],
    ["先手のコツⅡ", 175, 2, ],
    ["回避性能Ⅱ", 176, 3, ],
    ["漲るやる気", 177, 4, ],
    ["限界突破", 178, 8, ],

],

// スキルツリーの派生設定を行います。
// skillTreeDerivative: { ～ }の中にタイプ数分のスキルツリーの登録を行います。

// スキルツリーの派生設定は次のように行います。
// "タイプ名": [ [スキル1, [派生先スキル1, 派生先スキル2, ...]], [スキル2, [派生先スキル3, 派生先スキル4, ...] ]
// ※派生先スキルが存在しない終端スキルの場合、派生先スキルは省略可能です。
//
// 例えば、"様子を見る"と"連続攻撃"を取得すると、"２回攻撃"が取得できるようにするには、次の設定を行います。
// ["様子を見る", ["２回攻撃"]],
// ["連続攻撃", ["２回攻撃"]],
// ["２回攻撃"],
//
// また、"ヒール"を取得すると、"ファイア"と"スパーク"が取得できるようにするには、次の設定を行います。
// ["ヒール", ["ファイア"]],
// ["ヒール", ["スパーク"]],
// ["ファイア"],
// ["スパーク"],
skillTreeDerivative: {
    "アンのスキル": [
["強斬り", ["短剣術Ⅰ"]],
["強斬り", ["目覚め"]],
["強斬り", ["パワーシール"]],
["短剣術Ⅰ"],
["目覚め"],
["パワーシール"],
["短剣術Ⅰ", ["真空刃"]],
["真空刃", ["短剣術Ⅱ"]],
["短剣術Ⅱ", ["穿ち断ち"]],
["穿ち断ち", ["短剣術Ⅲ"]],
["短剣術Ⅲ", ["情動"]],
["情動"],

["目覚め", ["血刃"]],
["血刃", ["血の魔法"]],
["血の魔法", ["捨て身の構え"]],
["捨て身の構え", ["血の慟哭"]],
["血の慟哭", ["情動"]],
["情動"],

["目覚め", ["ダークⅠ"]],
["ダークⅠ", ["闇のマナⅠ"]],
["闇のマナⅠ", ["ダークⅡ"]],
["闇のマナⅠ", ["ダークオール"]],
["ダークオール"],
["ダークⅡ", ["闇のマナⅡ"]],
["闇のマナⅡ", ["ダークⅢ"]],
["ダークⅢ"],
["ダークⅠ", ["シェルブランド"]],
["ダークⅠ", ["ウィークウインド"]],
["シェルブランド", ["魔力再生"]],
["ウィークウインド", ["効率化"]],
["魔力再生", ["ハンドオブグレゴリア"]],
["効率化", ["ハンドオブグレゴリア"]],
["ハンドオブグレゴリア"],

["パワーシール", ["スピードシール"]],
["パワーシール", ["斥候術Ⅰ"]],
["スピードシール", ["ハンドオブグレゴリア"]],
["ハンドオブグレゴリア"],
["斥候術Ⅰ", ["斥候術Ⅱ"]],
["斥候術Ⅱ", ["ハンドオブグレゴリア"]],
["ハンドオブグレゴリア"],

    ],

    "ラィモネのスキル": [
["ファイアⅠ", ["火のマナⅠ"]],
["ファイアⅠ", ["サンダーⅠ"]],
["ファイアⅠ", ["ヒールⅠ"]],
["火のマナⅠ", ["ファイアⅡ"]],
["ファイアⅡ", ["火のマナⅡ"]],
["火のマナⅡ", ["火のマナⅢ"]],
["火のマナⅢ", ["ファイアⅢ"]],
["ファイアⅢ", ["ヴァーミリオンフレア"]],
["ヴァーミリオンフレア"],
["ファイアⅡ", ["ブレイブフォース"]],
["ブレイブフォース", ["魔術の心得Ⅰ"]],
["魔術の心得Ⅰ", ["魔術の心得Ⅱ"]],
["魔術の心得Ⅱ"],


["サンダーⅠ", ["雷のマナⅠ"]],
["雷のマナⅠ", ["サンダーⅡ"]],
["サンダーⅡ", ["雷のマナⅡ"]],
["雷のマナⅡ", ["雷のマナⅢ"]],
["雷のマナⅢ", ["サンダーⅢ"]],
["サンダーⅢ", ["ヴァーミリオンフレア"]],
["ヴァーミリオンフレア"],
["サンダーⅠ", ["チャージ"]],
["チャージ", ["魔術の心得Ⅲ"]],
["魔術の心得Ⅲ", ["魔術の心得Ⅳ"]],
["魔術の心得Ⅳ"],

["ヒールⅠ", ["光のマナⅠ"]],
["光のマナⅠ", ["光のマナⅡ"]],
["光のマナⅡ", ["ヒールⅡ"]],
["ヒールⅡ", ["光のマナⅢ"]],

["光のマナⅢ", ["リヴァイブ"]],
["光のマナⅢ", ["プリズムレイ"]],
["リヴァイブ"],
["プリズムレイ"],

["ヒールⅠ", ["アクアベール"]],
["アクアベール", ["水のマナⅠ"]],
["水のマナⅠ", ["ライフシャワー"]],
["ライフシャワー"],
["アクアベール", ["アイスブランド"]],
["アイスブランド", ["水のマナⅡ"]],
["水のマナⅡ"],


        ],
    "ミスティのスキル": [
["ドラゴンクロー", ["臨戦態勢Ⅰ"]],
["ドラゴンクロー", ["超再生"]],
["ドラゴンクロー", ["回避性能Ⅰ"]],
["臨戦態勢Ⅰ", ["ドラゴンバッシュ"]],
["ドラゴンバッシュ", ["臨戦態勢Ⅱ"]],
["臨戦態勢Ⅱ", ["龍の力"]],
["龍の力", ["ドラゴンラッシュ"]],
["ドラゴンラッシュ", ["激龍化"]],
["激龍化"],
["臨戦態勢Ⅰ", ["アイスブレス"]],
["アイスブレス", ["魔力増加"]],
["魔力増加", ["マナの権化"]],
["マナの権化", ["ミストブレイク"]],
["ミストブレイク", ["激龍化"]],
["激龍化"],
["超再生", ["活性Ⅰ"]],
["活性Ⅰ", ["ミストヒール"]],
["ミストヒール", ["活性Ⅱ"]],
["活性Ⅱ", ["龍の堅鱗"]],
["龍の堅鱗", ["マナキュアー"]],
["マナキュアー"],
["超再生", ["防御姿勢Ⅰ"]],
["防御姿勢Ⅰ", ["かばう"]],
["かばう", ["防御姿勢Ⅱ"]],
["防御姿勢Ⅱ", ["霧の衣"]],
["霧の衣", ["龍の守護"]],
["龍の守護"],

["回避性能Ⅰ", ["スタンテイル"]],
["スタンテイル", ["先手のコツⅠ"]],
["先手のコツⅠ", ["ミストファング"]],
["ミストファング", ["再生力"]],
["再生力", ["限界突破"]],
["限界突破"],
["回避性能Ⅰ", ["先手のコツⅡ"]],
["先手のコツⅡ", ["霧の領域"]],
["霧の領域", ["回避性能Ⅱ"]],
["回避性能Ⅱ", ["漲るやる気"]],
["漲るやる気", ["限界突破"]],
["限界突破"],







        ],
},

// レベルアップによってSPを獲得する場合、レベルごとに得られるSP値を以下の形式で設定します。
// classId: 職業ID, default: デフォルト値, レベル: SP値, レベル: SP値, ...
// 下記の設定例では、レベル2では3SP取得、レベル3では4SP取得、それ以外のレベルでは5SPを獲得します。
levelUpGainSp: [
    {
        classId: 1,
        default: 1,
 
    },

    {
        classId: 2,
        default: 1,

    },

    {
        classId: 3,
        default: 1,

    },




]
// =============================================================
// ●設定項目はここまでです。
// =============================================================
};
};
